@@ -25,11 +25,13 @@
 #include "includes.h"
 #include "uuencode.h"
 
-RCSID("$OpenBSD: radix.c,v 1.15 2001/01/16 23:58:09 deraadt Exp $");
+RCSID("$OpenBSD: radix.c,v 1.16 2001/06/23 15:12:19 itojun Exp $");
 
 #ifdef AFS
 #include <krb.h>
 
+#include <radix.h>
+
 typedef u_char my_u_char;
 typedef u_int my_u_int32_t;
 typedef u_short my_u_short;
